local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

function Capitalist:ApplyFilters()
	self:DebugEntry("ApplyFilters")

	-- reset memory
	self:ComponentMemoryReset()
	self:ProductMemoryReset()

	-- REVIEW: Enable seeing and setting counts for crafts by alts
	-- CONSIDER:  Automatic queueing of subcomponents that alts need to make

	-- iterate through each profession this character has	
	if (self.db.profile[realm] and self.db.profile[realm][character]) then
		for profession, professionData in pairs(self.db.profile[realm][character]) do
			self:DebugMisc(profession)

			for skillName, numCreated in pairs(self.db.profile[realm][character][profession]) do
				if (self:ProductIsEnabled(profession, skillName)) then
					local product = self.db.profile.professions[profession][skillName]

					-- analyze all components
					local componentBlocked = false
					for componentName, componentCount in pairs(product.components) do
						if (not self:IsComponentAllowed(componentName)) then
							componentBlocked = true
							break
						end
					end

					if (componentBlocked) then
						-- hide products with blocked components
						self:ProductSetVisible(profession, skillName, false)
						self:ProductComponentsSetVisible(profession, skillName, false)
					else
						for componentName, componentCount in pairs(product.components) do
							-- make visible components that are in use
							self:ComponentSetVisible(componentName, true)
						end
					end
				end
			end

			if (profession == L["Enchanting"]) then
				self:ComponentSetVisible(L["Enchanting Vellum"], true)
			end
		end
	end

	self:BuildList()

	self:DebugExit("ApplyFilters")
end

-- scans the trade skill frame and adds all products the player knows
function Capitalist:ScanTradeSkillFrame(...)
	self:DebugEntry("ScanTradeSkillFrame")

	local profession = ...;
	self:DebugMisc(profession)

	-- REVIEW: Update products for other players, and maybe track their names to do business later
	-- Bypass data updating when viewing another player's profession information
	if (IsTradeSkillLinked()) then
	 return
	end

	SetTradeSkillSubClassFilter(0, 1, 1)
	SetTradeSkillInvSlotFilter(0, 1, 1)

	-- REVIEW: This had errors for non-Enchanting professions
	--ExpandTradeSkillSubClass(1)

	SetTradeSkillItemNameFilter("")
	
	local newCharacterProducts = 0

	if (not self.db.profile[realm]) then
		self.db.profile[realm] = {}
	end
	if (not self.db.profile[realm][character]) then
		self.db.profile[realm][character] = {}
	end
	if (not self.db.profile[realm][character][profession]) then
		self.db.profile[realm][character][profession] = {}
	end

	local newProducts = 0
	local newComponents = 0

	if (not self.db.profile.professions) then
		self.db.profile.professions = {}
	end
	if (not self.db.profile.professions[profession]) then
		self.db.profile.professions[profession] = {}
	end
	
	if (not self.db.profile.components) then
		self.db.profile.components = {}
	end
	
	for skillId=1, GetNumTradeSkills() do
		local skillName, skillType, numAvailable, isExpanded, altVerb = GetTradeSkillInfo(skillId)
		self:DebugMisc(skillName)		-- "Enchant Bracers - Major Stamina"
		self:DebugMisc(skillType)		-- "header" or one of ("trivial", "easy", "medium", "optimal", "difficult")
		self:DebugMisc(numAvailable)	-- the bracketed value shown in the craft UI, showing how many you can make with current components
		self:DebugMisc(isExpanded)		--
		self:DebugMisc(altVerb)			-- nil or "Enchant" (REVIEW: perhaps others for armor kits?)

		local skillLink = GetTradeSkillItemLink(skillId)
		self:DebugMisc(skillLink)

		if (skillType ~= "header" and L.checkForValidSkillName(profession, skillName)) then
			local skillDescription = GetTradeSkillDescription(skillId)
			self:DebugMisc(skillDescription)

			local minMade, maxMade = GetTradeSkillNumMade(skillId)
			self:DebugMisc(minMade)
			self:DebugMisc(maxMade)

			if (maxMade == minMade) then
				maxMade = nil
				if (minMade == 1) then
					minMade = nil
				end
			end

			-- add it to the known products
			if (not self.db.profile.professions[profession][skillName]) then
				self.db.profile.professions[profession][skillName] = {
																		link = skillLink,
																		description = skillDescription,
																		components = {},
																	 }

				-- enchanting-specific
				if (profession == L["Enchanting"]) then
					self.db.profile.professions[profession][skillName].scroll = L.getScrollName(skillName)
				end

				newProducts = newProducts + 1
			end

			-- add the known products for this character
			if (not self.db.profile[realm][character][profession][skillName]) then
				self.db.profile[realm][character][profession][skillName] = 0
				newCharacterProducts = newCharacterProducts + 1
			end

			if (self.db.profile.professions[profession][skillName].min ~= minMade) then
				self.db.profile.professions[profession][skillName].min = minMade
			end
			if (self.db.profile.professions[profession][skillName].max ~= maxMade) then
				self.db.profile.professions[profession][skillName].max = maxMade
			end

--			REVIEW: Can't add this yet because the character profession entries are N=C pairs not N={count=C}
--			if (self.db.profile[realm][character][profession][skillName].type ~= skillType) then
--				self.db.profile[realm][character][profession][skillName].type = skillType
--			end

			self.db.profile.professions[profession][skillName].components = {}

			for reagentId=1, GetTradeSkillNumReagents(skillId) do
				local reagentName, reagentTexture, reagentCount, playerReagentCount = GetTradeSkillReagentInfo(skillId, reagentId)
				self:DebugMisc(reagentName)
				self:DebugMisc(reagentTexture)
				self:DebugMisc(reagentCount)
				self:DebugMisc(playerReagentCount)

				if (reagentName and (reagentCount > 0)) then
					self.db.profile.professions[profession][skillName].components[reagentName] = reagentCount

					-- for previously unknown components, save the item id
					if (not self.db.profile.components[reagentName]) then
						local reagentLink = GetTradeSkillReagentItemLink(skillId, reagentId)
						self:DebugMisc(reagentLink)

						local _, _, reagentItemId = reagentLink:find("Hitem:(%d+):")
						self:DebugMisc(reagentItemId)
					
						self.db.profile.components[reagentName] = {
																	id = tonumber(reagentItemId),
																  }

						newComponents = newComponents + 1
					end
				end
			end
		end
	end

	if (profession == L["Enchanting"]) then
		local componentName = L["Enchanting Vellum"]
		if (not self.db.profile.components[componentName]) then
			self.db.profile.components[componentName] = {
															id = self:GetComponentId(componentName),
														}
			newComponents = newComponents + 1
		end
	end
	
	if (newProducts + newComponents + newCharacterProducts > 0) then
		self:Print("found "..newProducts.." new products and "..newComponents.." new components")
		self:Print("found "..newCharacterProducts.." new character products")

		self:ApplyFilters(profession)
		self:UpdateFilterOptions()
	end
	
	self:UpdateRestockStack(profession)

	self:DebugExit("ScanTradeSkillFrame")
end

function Capitalist:BuildList()
	self:DebugEntry("BuildList")

	self.list = {}

	if (self.db.profile[realm] and self.db.profile[realm][character]) then
		for profession, professionData in pairs(self.db.profile[realm][character]) do
			self:DebugMisc(profession)

			for skillName, numCreated in pairs(self.db.profile[realm][character][profession]) do				
				local productName = skillName
				if ((profession == L["Enchanting"]) and
					self.db.profile.professions[profession] and
					self.db.profile.professions[profession][skillName]) then
					productName = self.db.profile.professions[profession][skillName].scroll
				end
				if (productName and
					self:ProductIsEnabled(profession, skillName) and
					self:ProductIsVisible(profession, skillName)) then
					self.list[productName] = { profession = profession, skillName = skillName }
				end
			end
		end
	end

	self:DebugExit("BuildList")
end
